@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2021 Michal Trojnara <Michal.Trojnara@stunnel.org>
+ *   Copyright (C) 1998-2023 Michal Trojnara <Michal.Trojnara@stunnel.org>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -186,7 +186,7 @@ typedef struct servername_list_struct SERVERNAME_LIST;/* forward declaration */
 
 #ifndef OPENSSL_NO_PSK
 typedef struct psk_keys_struct {
-    char *identity;
+    const char *identity;                  /* the OpenSSL API requires const */
     unsigned char *key_val;
     unsigned key_len;
     struct psk_keys_struct *next;
@@ -230,10 +230,13 @@ typedef struct service_options_struct {
     SOCK_OPT *sock_opts;
 
         /* service-specific data for verify.c */
-    char *ca_dir;                              /* directory for hashed certs */
-    char *ca_file;                       /* file containing bunches of certs */
-    char *crl_dir;                              /* directory for hashed CRLs */
-    char *crl_file;                       /* file containing bunches of CRLs */
+#ifndef OPENSSL_NO_ENGINE
+    NAME_LIST *ca_engine;  /* engine-specific CA certificate identifier list */
+#endif
+    char *ca_dir;                    /* directory containing hashed CA certs */
+    char *ca_file;                  /* file containing concatenated CA certs */
+    char *crl_dir;                       /* directory containing hashed CRLs */
+    char *crl_file;                     /* file containing concatenated CRLs */
 #ifndef OPENSSL_NO_OCSP
     char *ocsp_url;
     unsigned long ocsp_flags;
@@ -381,7 +384,7 @@ typedef union {
 } OPT_UNION;
 
 struct sock_opt_struct {
-    char *opt_str;
+    const char *opt_str;
     int  opt_level;
     int  opt_name;
     VAL_TYPE opt_type;
@@ -409,13 +412,9 @@ typedef struct {
     int main_thread;
 } s_poll_set;
 
-typedef struct disk_file {
-#ifdef USE_WIN32
-    HANDLE fh;
-#else
-    int fd;
-#endif
-    /* the interface is prepared to easily implement buffering if needed */
+typedef struct {
+    /* stdio is currently used, but alternative implementations can be added */
+    FILE *f;
 } DISK_FILE;
 
     /* definitions for client.c */
@@ -455,6 +454,7 @@ typedef struct client_data_struct {
     FD remote_fd;                                  /* remote file descriptor */
     unsigned long pid;                           /* PID of the local process */
     SOCKET fd;                                  /* temporary file descriptor */
+    int fatal_alert;                               /* received a fatal alert */
     RENEG_STATE reneg_state;         /* used to track renegotiation attempts */
     unsigned long long seq;          /* sequential thread number for logging */
     unsigned rr;    /* per-client sequential number for round-robin failover */
@@ -510,10 +510,10 @@ void service_free(SERVICE_OPTIONS *);
 #ifndef USE_FORK
 void get_limits(void); /* setup global max_clients and max_fds */
 #endif
-SOCKET s_socket(int, int, int, int, char *);
-int s_pipe(int[2], int, char *);
-int s_socketpair(int, int, int, SOCKET[2], int, char *);
-SOCKET s_accept(SOCKET, struct sockaddr *, socklen_t *, int, char *);
+SOCKET s_socket(int, int, int, int, const char *);
+int s_pipe(int[2], int, const char *);
+int s_socketpair(int, int, int, SOCKET[2], int, const char *);
+SOCKET s_accept(SOCKET, struct sockaddr *, socklen_t *, int, const char *);
 void set_nonblock(SOCKET, unsigned long);
 
 /**************************************** prototypes for log.c */
@@ -521,6 +521,8 @@ void set_nonblock(SOCKET, unsigned long);
 #define SINK_SYSLOG 1
 #define SINK_OUTFILE 2
 
+extern DISK_FILE *outfile;
+
 int log_open(int);
 void log_close(int);
 void log_flush(LOG_MODE);
@@ -530,8 +532,9 @@ void s_log(int, const char *, ...)
 #else
     ;
 #endif
+void s_vlog(int, const char *, va_list);
 char *log_id(CLI *);
-void fatal_debug(char *, const char *, int) NORETURN;
+void fatal_debug(const char *, const char *, int) NORETURN;
 #define fatal(a) fatal_debug((a), __FILE__, __LINE__)
 void ioerror(const char *);
 void sockerror(const char *);
@@ -550,7 +553,8 @@ DH *get_dh2048(void);
 /**************************************** prototypes for cron.c */
 
 #ifdef USE_OS_THREADS
-extern THREAD_ID cron_thread_id;
+extern THREAD_ID per_second_thread_id;
+extern THREAD_ID per_day_thread_id;
 #endif
 
 int cron_init(void);
@@ -560,7 +564,9 @@ int cron_init(void);
 extern int index_ssl_cli, index_ssl_ctx_opt;
 extern int index_session_authenticated, index_session_connect_address;
 
-int fips_available();
+int fips_default(void);
+int fips_available(void);
+void crypto_init(void);
 int ssl_init(void);
 int ssl_configure(GLOBAL_OPTIONS *);
 
@@ -582,12 +588,15 @@ PSK_KEYS *psk_find(const PSK_TABLE *, const char *);
 UI_METHOD *ui_stunnel(void);
 #endif /* !defined(OPENSSL_NO_ENGINE) */
 void print_session_id(SSL_SESSION *);
-void sslerror(char *);
+void sslerror(const char *);
 
 /**************************************** prototypes for verify.c */
 
 int verify_init(SERVICE_OPTIONS *);
-void print_client_CA_list(const STACK_OF(X509_NAME) *);
+#ifndef OPENSSL_NO_ENGINE
+X509 *engine_get_cert(ENGINE *, const char *);
+#endif
+void print_CA_list(const char *, const STACK_OF(X509_NAME) *);
 char *X509_NAME2text(X509_NAME *);
 
 /**************************************** prototypes for network.c */
@@ -678,17 +687,18 @@ extern HWND capwin_hwnd;
 extern LONG capwin_connectivity;
 #endif
 
-char *protocol(CLI *, SERVICE_OPTIONS *opt, const PHASE);
+const char *protocol(CLI *, SERVICE_OPTIONS *opt, const PHASE);
 
 /**************************************** prototypes for resolver.c */
 
-void resolver_init();
+void resolver_init(void);
+int use_ipv6(void);
 
 unsigned name2addr(SOCKADDR_UNION *, char *, int);
-unsigned hostport2addr(SOCKADDR_UNION *, char *, char *, int);
+unsigned hostport2addr(SOCKADDR_UNION *, char *, const char *, int);
 
 unsigned name2addrlist(SOCKADDR_LIST *, char *);
-unsigned hostport2addrlist(SOCKADDR_LIST *, char *, char *);
+unsigned hostport2addrlist(SOCKADDR_LIST *, char *, const char *);
 
 void addrlist_clear(SOCKADDR_LIST *, int);
 unsigned addrlist_dup(SOCKADDR_LIST *, const SOCKADDR_LIST *);
@@ -708,21 +718,8 @@ const char *s_gai_strerror(int);
 #endif
 
 #ifdef USE_WIN32
-
 /* rename some locally shadowed declarations */
 #define getnameinfo     local_getnameinfo
-
-#ifndef _WIN32_WCE
-typedef int (CALLBACK * GETADDRINFO) (const char *,
-    const char *, const struct addrinfo *, struct addrinfo **);
-typedef void (CALLBACK * FREEADDRINFO) (struct addrinfo *);
-typedef int (CALLBACK * GETNAMEINFO) (const struct sockaddr *, socklen_t,
-    char *, size_t, char *, size_t, int);
-extern GETADDRINFO s_getaddrinfo;
-extern FREEADDRINFO s_freeaddrinfo;
-extern GETNAMEINFO s_getnameinfo;
-#endif /* ! _WIN32_WCE */
-
 #endif /* USE_WIN32 */
 
 int getnameinfo(const struct sockaddr *, socklen_t,
@@ -830,12 +827,14 @@ void ignore_value(void *);
 /**************************************** prototypes for file.c */
 
 #ifndef USE_WIN32
-DISK_FILE *file_fdopen(int);
+DISK_FILE *file_fdopen(int, FILE_MODE mode);
 #endif
 DISK_FILE *file_open(char *, FILE_MODE mode);
 void file_close(DISK_FILE *);
 ssize_t file_getline(DISK_FILE *, char *, int);
-ssize_t file_putline(DISK_FILE *, char *);
+ssize_t file_putline_nonewline(DISK_FILE *, char *);
+ssize_t file_putline_newline(DISK_FILE *, char *);
+int file_flush(DISK_FILE *);
 int file_permissions(const char *);
 
 #ifdef USE_WIN32
@@ -845,18 +844,18 @@ LPSTR tstr2str(LPCTSTR);
 
 /**************************************** prototypes for libwrap.c */
 
-int libwrap_init();
+int libwrap_init(void);
 void libwrap_auth(CLI *);
 
 /**************************************** prototypes for tls.c */
 
 extern volatile int tls_initialized;
 
-void tls_init();
-TLS_DATA *tls_alloc(CLI *, TLS_DATA *, char *);
-void tls_cleanup();
+void tls_init(void);
+TLS_DATA *tls_alloc(CLI *, TLS_DATA *, const char *);
+void tls_cleanup(void);
 void tls_set(TLS_DATA *);
-TLS_DATA *tls_get();
+TLS_DATA *tls_get(void);
 
 /**************************************** prototypes for str.c */
 
@@ -868,11 +867,12 @@ struct tls_data_struct {
     size_t alloc_bytes, alloc_blocks;
     CLI *c;
     SERVICE_OPTIONS *opt;
-    char *id;
+    const char *id;
 };
 
-void str_init(TLS_DATA *);
-void str_cleanup(TLS_DATA *);
+void str_init(void);
+void str_thread_init(TLS_DATA *);
+void str_thread_cleanup(TLS_DATA *);
 char *str_dup_debug(const char *, const char *, int);
 #define str_dup(a) str_dup_debug((a), __FILE__, __LINE__)
 char *str_dup_detached_debug(const char *, const char *, int);
@@ -888,21 +888,29 @@ char *str_printf(const char *, ...)
 LPTSTR str_tprintf(LPCTSTR, ...);
 #endif
 
-void str_canary_init();
-void str_stats();
+void str_canary_init(void);
+void str_stats(void);
+
 void *str_alloc_debug(size_t, const char *, int);
 #define str_alloc(a) str_alloc_debug((a), __FILE__, __LINE__)
 void *str_alloc_detached_debug(size_t, const char *, int);
 #define str_alloc_detached(a) str_alloc_detached_debug((a), __FILE__, __LINE__)
+
 void *str_realloc_debug(void *, size_t, const char *, int);
 #define str_realloc(a, b) str_realloc_debug((a), (b), __FILE__, __LINE__)
 void *str_realloc_detached_debug(void *, size_t, const char *, int);
 #define str_realloc_detached(a, b) str_realloc_detached_debug((a), (b), __FILE__, __LINE__)
+
 void str_detach_debug(void *, const char *, int);
 #define str_detach(a) str_detach_debug((a), __FILE__, __LINE__)
+void str_detach_const_debug(const void *, const char *, int);
+#define str_detach_const(a) str_detach_const_debug((a), __FILE__, __LINE__)
+
 void str_free_debug(void *, const char *, int);
 #define str_free(a) str_free_debug((a), __FILE__, __LINE__), (a)=NULL
 #define str_free_expression(a) str_free_debug((a), __FILE__, __LINE__)
+void str_free_const_debug(const void *, const char *, int);
+#define str_free_const(a) str_free_const_debug((a), __FILE__, __LINE__), (a)=NULL
 
 void leak_table_utilization(void);
 
